require "nvchad.mappings"
local map = vim.keymap.set
local M = {}
M.General = function()
  map("n", ";", ":", { desc = "general enter command mode", nowait = true })
  map("v", ">", ">gv", { desc = "general indent" })
  map("n", "<C-Left>", "<C-w>h", { desc = "switch Window left" })
  map("n", "<C-Right>", "<C-w>l", { desc = "switch Window right" })
  map("n", "<C-Down>", "<C-w>j", { desc = "switch Window down" })
  map("n", "<C-Up>", "<C-w>k", { desc = "switch Window up" })
  map("n", "K", "<cmd>Lspsaga hover_doc<CR>", { desc = "LSPSaga Hover" })
  map("n", "]d", function()
    vim.diagnostic.goto_next()
  end, { desc = "LSP Next Diagnostic" })
  map("n", "[d", function()
    vim.diagnostic.goto_prev()
  end, { desc = "LSP Prev Diagnostic" })
  map("n", "<Leader>lf", "<cmd>Lspsaga show_cursor_diagnostics<CR>", { desc = "LSPSaga floating diagnostic" })
  map("n", "<Leader>gr", "<cmd>Lspsaga finder ref<CR>", { desc = "LSPSaga search references" })
  map("n", "<Leader>gi", "<cmd>Lspsaga finder imp<CR>", { desc = "LSPSaga search implementation" })
  map("n", "<Leader>gd", "<cmd>Lspsaga finder def<CR>", { desc = "LSPSaga search definition" })
  map("n", "<Leader>rr", "<cmd>Lspsaga rename<CR>", { desc = "LSPSaga rename" })
  map("n", "<Leader>ci", "<cmd>Lspsaga incoming_calls<CR>", { desc = "LSPSaga incoming calls" })
  map("n", "<Leader>co", "<cmd>Lspsaga outgoing_calls<CR>", { desc = "LSPSaga outgoing calls" })
  map("n", "<C-b>", "<cmd>Lspsaga outline<CR>", { desc = "LSPSaga outline" })
end

M.Ufo = function()
  map("n", "zR", require("ufo").openAllFolds, { desc = "UFO Open All Folds" })
  map("n", "zM", require("ufo").closeAllFolds, { desc = "UFO Close All Folds" })
end

M.Dap = function()
  map("n", "<leader>db", "<cmd> DapToggleBreakpoint <CR>", { desc = "dap Toggle Breakpoint" })
  map("n", "<leader>dus", function()
    local widgets = require "dap.ui.widgets"
    local sidebar = widgets.sidebar(widgets.scopes)
    sidebar.open()
  end, { desc = "dap Open debugging sidebar" })
  map("n", "<F5>", function()
    require("dap").continue()
  end, { desc = "dap Debug Program" })
  map("n", "<F10>", function()
    require("dap").step_over()
  end, { desc = "dap Step Over" })
  map("n", "<F11>", function()
    require("dap").step_into()
  end, { desc = "dap Step Into" })
  map("n", "<F12>", function()
    require("dap").step_out()
  end, { desc = "dap Step Out" })
  map("n", "<Leader>lp", function()
    require("dap").set_breakpoint(nil, nil, vim.fn.input "Log point message: ")
  end, { desc = "dap Set Breakpoint with Mesage?" })
  map("n", "<Leader>dr", function()
    require("dap").repl.open()
  end, { desc = "dap Open REPL" })
  map("n", "<Leader>dl", function()
    require("dap").run_last()
  end, { desc = "dap Run Last" })
  map("n", "<Leader>dp", function()
    require("dap.ui.widgets").preview()
  end, { desc = "dap Preview UI" })
  map("n", "<Leader>dh", function()
    require("dap.ui.widgets").hover()
  end, { desc = "dap Hover UI" })
  map("n", "<Leader>do", function()
    require("dapui").toggle()
  end, { desc = "dap Toggle Debug UI" })
  map("n", "<Leader>df", function()
    local widgets = require "dap.ui.widgets"
    widgets.centered_float(widgets.frames)
  end, { desc = "dap Open Frames" })
  map("n", "<Leader>ds", function()
    local widgets = require "dap.ui.widgets"
    widgets.centered_float(widgets.scopes)
  end, { desc = "dap Open Scopes" })
end
M.Dap_python = function()
  map("n", "<leader>dpr", function()
    require("dap-python").test_method()
  end, { desc = "dap_python Test Method" })
end
M.Dap_go = function()
  map("n", "<leader>dpr", function()
    require("dap-go").debug_test()
  end, { desc = "dap_go Test Method" })
end
M.Crates = function()
  map("n", "<leader>rcu", function()
    require("crates").upgrade_all_crates()
  end, { desc = "crates update crates" })
end
M.LazyGit = function()
  map("n", "<leader>gg", function()
    vim.cmd ":LazyGit"
  end, { desc = "lazygit open lazygit" })
end
M.Hop = function()
  map("n", "t", function()
    require("hop").hint_words { multi_windows = true }
  end, { desc = "hop Hop across windows", remap = true })
  map("n", "T", function()
    require("hop").hint_char2 { multi_windows = true }
  end, {
    desc = "hop Hop to chars across windows",
    remap = true,
  })
end
M.Copilot = function()
  map("i", "<M-l>", function()
    require("copilot.suggestion").accept()
  end, { desc = "copilot Accept Suggestion", "opts = { remap = true }" })
  map("i", "<M-)>", function()
    require("copilot.suggestion").next()
  end, { desc = "copilot Next Suggestion", "opts = { remap = true }" })
  map("i", "<M-map(>", function()
    require("copilot.suggestion").prev()
  end, { desc = "copilot rev Suggestion", "opts = { remap = true }" })
  map("i", "<C-)>", function()
    require("copilot.suggestion").dismiss()
  end, { desc = "copilot Dismiss Suggestion", "opts = { remap = true }" })
end
M.Markdown = function()
  map("n", "<leader>cb", function()
    require("markdown-togglecheck").toggle()
  end, { desc = "markdown Toggle Checkmark" })
end
M.Obsidian = function()
  map("n", "gf", function()
    return require("obsidian").util.gf_passthrough()
  end, { desc = "obsidian Follow Link", "opts = { noremap = false, expr = true, buffer = true }" })
  map("n", "<leader>cb", function()
    return require("obsidian").util.toggle_checkbox()
  end, { desc = "obsidian Toggle Checkmark", "opts = { buffer = trueue, remap = true }" })
end
return M
